@@ -1,4 +1,4 @@
-/* Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2003, 2004 Thorsten Kukuk
+/* Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2003, 2004, 2006, 2008 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -38,10 +38,8 @@
 #include "compat.h"
 
 int dns_flag = 0;
-#if USE_SLP
 int slp_flag = 0;
 unsigned long int slp_timeout = 3600;
-#endif
 int xfr_check_port = 0;
 char *trusted_master = NULL;
 /* cached_filehandles (how many databases will be cached):
@@ -199,7 +197,12 @@ load_ypserv_conf (const char *path)
 	    size_t i, j;
 	    unsigned long files = 30;
 
-	    fgets (buf1, sizeof (buf1) - 1, in);
+	    if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
+
 	    i = 0;
 	    buf1[sizeof (buf1) - 1] = '\0';
 	    while (c != ':' && i <= strlen (buf1))
@@ -244,7 +247,12 @@ load_ypserv_conf (const char *path)
 	  {
 	    size_t i, j;
 
-	    fgets (buf1, sizeof (buf1) - 1, in);
+	    if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
+
 	    i = 0;
 	    while (c != ':' && i <= strlen (buf1))
 	      {
@@ -293,7 +301,12 @@ load_ypserv_conf (const char *path)
 	  {			/* sunos_kludge / slp */
 	    size_t i;
 
-	    fgets (buf1, sizeof (buf1) - 1, in);
+	    if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
+
 	    i = 0;
 	    while (c != ':' && i <= strlen (buf1))
 	      {
@@ -316,7 +329,6 @@ load_ypserv_conf (const char *path)
 
 	    if ((buf1[i - 1] == ':') && (strcasecmp (buf2, "slp") == 0))
 	      {
-#if USE_SLP
 		size_t j;
 
 		while (((buf1[i] == ' ') || (buf1[i] == '\t')) &&
@@ -340,15 +352,15 @@ load_ypserv_conf (const char *path)
 
 		if (debug_flag)
 		  log_msg ("ypserv.conf: slp: %d", slp_flag);
-#else
-		log_msg ("Support for SLP (line %d) is not compiled in.",
-			 line);
+#if !USE_SLP
+		if (slp_flag != 0)
+		  log_msg ("Support for SLP (line %d) is not compiled in.",
+			   line);
 #endif
 	      }
 	    else if ((buf1[i - 1] == ':') &&
 		     (strcasecmp (buf2, "slp_timeout") == 0))
 	      {
-#if USE_SLP
 		size_t j;
 
 		while (((buf1[i] == ' ') || (buf1[i] == '\t')) &&
@@ -363,11 +375,6 @@ load_ypserv_conf (const char *path)
 
 		if (debug_flag)
 		  log_msg ("ypserv.conf: slp_timeout: %lu", slp_timeout);
-
-#else
-		log_msg ("Support for SLP (line %d) is not compiled in.",
-			 line);
-#endif
 	      }
 	    else
 	      log_msg ("Parse error in line %d: => Ignore line", line);
@@ -379,7 +386,12 @@ load_ypserv_conf (const char *path)
 	  {			/* tryresolve / trusted_master */
 	    size_t i, j;
 
-	    fgets (buf1, sizeof (buf1) - 1, in);
+	    if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
+
 	    i = 0;
 	    while (c != ':' && i <= strlen (buf1))
 	      {
@@ -426,7 +438,12 @@ load_ypserv_conf (const char *path)
 	  {			/* xfr_check_port */
 	    size_t i, j;
 
-	    fgets (buf1, sizeof (buf1) - 1, in);
+	    if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
+
 	    i = 0;
 	    while (c != ':' && i <= strlen (buf1))
 	      {
@@ -476,7 +493,11 @@ load_ypserv_conf (const char *path)
 	    conffile_t *tmp;
 
 	    buf1[0] = c;
-	    fgets (&buf1[1], sizeof (buf1) - 2, in);
+	    if (fgets (&buf1[1], sizeof (buf1) - 2, in) == NULL)
+	      {
+		log_msg ("Read error in line %d => Ignore line", line);
+		break;
+	      }
 
 	    n = strtok (buf1, ":");
 	    if (n == NULL)
@@ -598,10 +619,11 @@ load_ypserv_conf (const char *path)
 	case '\n':
 	  break;		/* Ignore newline */
 	case '#':
-	  fgets (buf1, sizeof (buf1) - 1, in);
+	  if (fgets (buf1, sizeof (buf1) - 1, in) == NULL)
+	    log_msg ("Read error in line %d => Ignore line", line);
 	  break;
 	default:
-	  fgets (buf1, sizeof (buf1) - 1, in);
+	  if (fgets (buf1, sizeof (buf1) - 1, in) == NULL) {};
 	  log_msg ("Parse error in line %d: %c%s", line, c, buf1);
 	  break;
 	}
