@@ -1,4 +1,4 @@
-/* Copyright (c) 1999, 2000, 2001, 2005 Thorsten Kukuk
+/* Copyright (c) 1999, 2000, 2001, 2005, 2006, 2010 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -80,7 +80,7 @@ char *path_shadow_old = NULL;
 char *external_update_program = NULL;
 
 static int external_update_env (yppasswd *yppw);
-static int external_update_pipe (yppasswd *yppw);
+static int external_update_pipe (yppasswd *yppw, char *logbuf);
 static int update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 			 int *passwd_changed, int *chfn, int *chsh);
 
@@ -271,7 +271,7 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 
       if (x_flag)
         {
-          res = external_update_pipe (yppw);
+          res = external_update_pipe (yppw, logbuf);
           return &res;
         }
       else
@@ -415,7 +415,14 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       return 1;
     }
   chmod (path_passwd_tmp, passwd_stat.st_mode);
-  chown (path_passwd_tmp, passwd_stat.st_uid, passwd_stat.st_gid);
+  if (chown (path_passwd_tmp, passwd_stat.st_uid, passwd_stat.st_gid) == -1)
+    {
+      log_msg ("chown failed: %s", strerror (errno));
+      fclose (oldpf);
+      fclose (newpf);
+      unlink (path_passwd_tmp);
+      return 1;
+    }
 
 #ifdef HAVE_GETSPNAM
   /* Open the shadow file for reading. */
@@ -432,15 +439,26 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 
       if ((newsf = fopen (path_shadow_tmp, "w+")) == NULL)
 	{
+	  int err = errno;
 	  log_msg ("%s failed", logbuf);
-	  log_msg ("Can't open %s.tmp: %m", path_passwd);
+	  log_msg ("Can't open %s.tmp: %s",
+		   path_passwd, strerror (err));
 	  fclose (oldsf);
 	  fclose (newpf);
 	  fclose (oldpf);
 	  return 1;
 	}
       chmod (path_shadow_tmp, shadow_stat.st_mode);
-      chown (path_shadow_tmp, shadow_stat.st_uid, shadow_stat.st_gid);
+      if (chown (path_shadow_tmp, shadow_stat.st_uid,
+		 shadow_stat.st_gid) == -1)
+	{
+	  log_msg ("chown failed", strerror (errno));
+	  fclose (newsf);
+	  fclose (oldsf);
+	  fclose (newpf);
+	  fclose (oldpf);
+	  return 1;
+	}
     }
 #endif /* HAVE_GETSPNAM */
 
@@ -598,8 +616,10 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       /* write the passwd entry to tmp file */
       if (putpwent (pw, newpf) < 0)
 	{
+	  int err = errno;
 	  log_msg ("%s failed", logbuf);
-	  log_msg ("Error while writing new password file: %m");
+	  log_msg ("Error while writing new password file: %s",
+		   strerror (err));
 	  *passwd_changed = 0;
 	  break;
 	}
@@ -623,7 +643,9 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   if (*shadow_changed)
     {
       unlink (path_shadow_old);
-      link (path_shadow, path_shadow_old);
+      if (link (path_shadow, path_shadow_old) == -1)
+	log_msg ("Cannot create backup file %s: %s",
+		 path_shadow_old, strerror (errno));
       rename (path_shadow_tmp, path_shadow);
     }
   else
@@ -633,7 +655,9 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   if (*passwd_changed)
     {
       unlink (path_passwd_old);
-      link (path_passwd, path_passwd_old);
+      if (link (path_passwd, path_passwd_old) == -1)
+	log_msg ("Cannot create backup file %s: %s",
+		 path_passwd_old, strerror (errno));
       rename (path_passwd_tmp, path_passwd);
     }
   else
@@ -718,10 +742,9 @@ external_update_env (yppasswd *yppw)
  *===============================================================*/
 
 static int
-external_update_pipe (yppasswd *yppw)
+external_update_pipe (yppasswd *yppw, char *logbuf)
 {
   struct xpasswd *newpw;       /* passwd struct passed by the client */
-  char *logbuf;
   int res, pid, tochildpipe[2], toparentpipe[2];
   FILE *fp;
   char childresponse[1024];
@@ -737,12 +760,7 @@ external_update_pipe (yppasswd *yppw)
 
   newpw = &yppw->newpw;
   res = 1;
-  logbuf = alloca (60 + strlen (yppw->newpw.pw_name));
-  if (logbuf == NULL)
-    {
-      log_msg ("rpc.yppasswdd: out of memory");
-      return res;
-    }
+
   /*
    * determine what information we have to change
    */
@@ -754,7 +772,7 @@ external_update_pipe (yppasswd *yppw)
     shell = newpw->pw_shell;
 
   if (allow_chfn && newpw->pw_gecos && *(newpw->pw_gecos))
-    gcos = newpw->pw_shell;
+    gcos = newpw->pw_gecos;
 
   if (!password && !shell && !gcos)
     {
@@ -852,7 +870,7 @@ external_update_pipe (yppasswd *yppw)
       return res;
     }
 
-  strcat (parentmsg, yppw->newpw.pw_name);
+  strcpy (parentmsg, yppw->newpw.pw_name);
   strcat (parentmsg, " o:");
   strcat (parentmsg, yppw->oldpass);
   strcat (parentmsg, " ");
@@ -884,7 +902,6 @@ external_update_pipe (yppasswd *yppw)
   fp = fdopen(tochildpipe[1], "w");
   fprintf(fp, "%s\n", parentmsg);
   fclose(fp);
-  free (parentmsg);
 
   /*
    * get output from the child
@@ -902,11 +919,13 @@ external_update_pipe (yppasswd *yppw)
     {
       log_msg ("%s failed.  Change request: %s", logbuf, parentmsg);
       log_msg ("Response was %s", childresponse);
+      free (parentmsg);
       return res;
     }
 
   log_msg ("%s successful. Change request: %s", logbuf, parentmsg);
 
+  free (parentmsg);
   res = 0;
 
   return res;
